''
'' full colour VGA (4 pixels/hub window) - demo feeder, experimental
''
''        Author: Marko Lukat
'' Last modified: 2011/12/10
''       Version: 0.2
''
'' long[par][0]: double (scan line) buffer -> zero (ready)
'' long[par][1]: writer mailbox
'' long[par][2]: banner location
'' long[par][3]: unused
''
OBJ
  system: "core.con.system"
  
VAR
  long  food[4]
  long  eins[lcnt], zwei[lcnt]
  
PUB null
'' This is not a top level object.

PUB init(ID, mailbox)

  food{0} := @zwei{0} >< 32 | @eins{0}
  food[1] := mailbox
  food[2] := @banner
  
  return system.launch(ID, @feeder, @food{0})

DAT             org     0                       ' cog binary header

header_2048     long    system#ID_2             ' magic number for a cog binary
                word    header_size             ' header size
                word    %00000000_00000000      ' flags
                word    0, 0                    ' start register, register count

header_size     fit     16
                
DAT             org     0                       ' noise feeder

feeder          add     temp, par               ' @long[par][1]
                rdword  temp, temp              ' target mailbox

                add     hram, par               ' @long[par][2]
                rdword  hram, hram              ' banner location

                rdlong  scan, par               ' double buffer
                wrlong  zero, par               ' up and running
                
                add     sync, temp              ' frame indicator
                add     link, temp              ' writer control (secondary)
                rev     link, #{32-}0
                add     link, temp              ' writer control (primary)

' The writer (secondary pixel generator) writes its next waitcnt target to the frame
' indicator just before it starts with the vertical blank sequence. This value differs
' at least in bit 0 from a sample taken immediately after a hubop (4n).

                mov     rcnt, cnt               ' |
                wrlong  rcnt, sync              ' |
                rdlong  temp, sync              ' |
                cmp     rcnt, temp wz           ' |
        if_e    jmp     #$-2                    ' sync to vblank
        
' This is just a simple demo. We draw a banner image from hub and noise
' for the remainder.

screen          mov     rcnt, #hcnt             ' line count
                mov     data, hram              ' reset banner address

:eins           rdlong  temp, link wz           ' buffer slot empty?
        if_nz   jmp     #$-1                    ' try again

                mov     addr, scan              ' reset buffer address
                mov     ecnt, #lcnt             ' element count

:zwei           rdlong  temp, data              ' |
                add     data, #4                ' |
                and     temp, mask_import       ' retrieve and process image data
                
                wrlong  temp, addr              ' |
                add     addr, #4                ' |
                djnz    ecnt, #:zwei            ' fill scan buffer with image data

                wrlong  scan, link              ' displace buffer

                rev     scan, #{32-}0           ' |
                rev     link, #{32-}0           ' swap buffers and slots

                djnz    rcnt, #:eins            ' repeat

' ... now for the noise

noise           mov     rcnt, #300 - hcnt +1{scroll}

:drei           rdlong  temp, link wz           ' buffer slot empty?
        if_nz   jmp     #$-1                    ' try again

                mov     addr, scan              ' reset buffer address
                mov     ecnt, #lcnt             ' element count

:vier           shl     lfsr, #1 wc             ' |
        if_c    xor     lfsr, signature         ' iterate LFSR

                mov     temp, lfsr              ' grab a copy of the LFSR
                and     temp, mask              ' stay clear of sync bits

                wrlong  temp, addr              ' |
                add     addr, #4                ' |
                djnz    ecnt, #:vier            ' fill scan buffer with noise

                wrlong  scan, link              ' displace buffer

                rev     scan, #{32-}0           ' |
                rev     link, #{32-}0           ' swap buffers and slots

                djnz    rcnt, #:drei            ' repeat

' sort out scroll direction

setup           add     locn, #1                ' update current line

                cmp     locn, #300 - hcnt wz
        if_z    sub     noise, #2               ' +1 => -1
        if_z    movi    setup, #%100001_001     ' sub #

                cmp     locn, #0 wz
        if_z    add     noise, #2               ' -1 => +1
        if_z    movi    setup, #%100000_001     ' add #
        
                jmp     #screen                 ' next frame

' initialised data and/or presets
                
mask_import     long    $FCFCFCFC               ' |
mask            long    $54545454               ' stay clear of sync bits

lfsr            long    42                      ' |
signature       long    $1D872B41               ' noise generator

sync            long    12                      ' frame indicator
link            long    4 >< 32 | 8             ' primary/secondary buffer slots
locn            long    0                       ' current line
hram            long    8                       ' banner location
temp            long    4                       ' 

' uninitialised data and/or temporaries

scan            res     1                       ' primary/secondary buffer addresses
addr            res     1                       ' current buffer address (during fill)
data            res     1                       ' image data

ecnt            res     1                       ' element count
rcnt            res     1                       '     row count

                fit

CON
  zero = $1F0                                   ' par (dst only)
  lcnt =  100                                   ' longs per scan line
  hcnt =   75                                   ' hub image line count
  
DAT banner      long    'kuroneko.vga'
                file    "kuroneko.vga"
{
                long    $F05050F0[100]
                long    $F0F05050[100]
                long    $50F0F050[100]

                long    0[6900]

                long    $F05050F0[100]
                long    $F0F05050[100]
                long    $50F0F050[100]
}
DAT